/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.crov.prase.controller.prase;

import com.arrebol.taxiservicios.controller.connection.ConnectionManager;
import com.arrebol.taxiservicios.controller.util.HibernateUtil;
import com.arrebol.taxiservicios.model.catalog.Location;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.enums.GenericEnumType;
import com.crov.prase.model.prase.DetellePagoPoliza;
import com.crov.prase.model.prase.PagosPoliza;
import com.crov.prase.model.prase.Poliza;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import javax.persistence.NoResultException;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 *
 * @author Crov
 */
public class PagosPolizaController extends ConnectionManager implements Serializable {

   private static final long serialVersionUID = -8772793021819350069L;
   final Logger logger = LogManager.getLogger(getClass());

   public List<PagosPoliza> fillPagosPolizaByPoliza(Poliza poliza) {
      logger.info("fillPagosPolizaByPoliza");
      List<PagosPoliza> resultList = null;
      Transaction transaction = null;
      try {
         logger.info("Searching pagos for poliza..");
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         CriteriaBuilder builder = session.getCriteriaBuilder();

         CriteriaQuery<PagosPoliza> criteria = builder.createQuery(PagosPoliza.class);
         Root<PagosPoliza> root = criteria.from(PagosPoliza.class);

         Predicate criterio1 = builder.equal(root.get("poliza"), poliza);
         Predicate criterio2 = builder.equal(root.get("estatusActivo"), GenericEnumType.ENABLED);
         criteria.where(builder.and(criterio1, criterio2));
         criteria.orderBy(builder.asc(root.get("fechaAPagar")));

         resultList = session.createQuery(criteria).getResultList();

         logger.info("Total of pagos found: " + (null == resultList ? -1 : resultList.size()));
         transaction.commit();
      } catch (HibernateException e) {
         logger.error("Pagos cannot be loaded ");
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method fillPagosPolizaByPoliza ", e);
         rollback(transaction);
      }
      return resultList;
   }

   public List<PagosPoliza> fillPagosPolizaFechaAPagarByLocationFechaInicioAndFechaFin(Location location, Date startDate, Date endDate) {
      logger.info("fillPagosPolizaByPoliza");
      List<PagosPoliza> resultList = null;
      Transaction transaction = null;
      try {
         logger.info("Searching pagos for poliza..");
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         CriteriaBuilder builder = session.getCriteriaBuilder();

         CriteriaQuery<PagosPoliza> criteria = builder.createQuery(PagosPoliza.class);
         Root<PagosPoliza> root = criteria.from(PagosPoliza.class);

         LocalDate localStartDate = startDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
         LocalDate localEndDate = endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();

         Predicate criterio1 = builder.equal(root.get("poliza").get("location"), location);
         Predicate criterio2 = builder.equal(root.get("estatusActivo"), GenericEnumType.ENABLED);
         Predicate criterio3 = builder.equal(root.get("pagoEstatus"), GenericEnumType.DISABLED);
         Predicate startDatePredicate = builder.greaterThanOrEqualTo(root.get("fechaAPagar").as(LocalDate.class), localStartDate);
         Predicate endDatePredicate = builder.lessThanOrEqualTo(root.get("fechaAPagar").as(LocalDate.class), localEndDate);
         criteria.where(builder.and(criterio1, criterio2, criterio3, startDatePredicate, endDatePredicate));
         criteria.orderBy(builder.asc(root.get("fechaAPagar")));

         resultList = session.createQuery(criteria).getResultList();

         logger.info("Total of pagos found: " + (null == resultList ? -1 : resultList.size()));
         transaction.commit();
      } catch (HibernateException e) {
         logger.error("Pagos cannot be loaded ");
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method fillPagosPolizaByPoliza ", e);
         rollback(transaction);
      }
      return resultList;
   }

   public PagosPoliza fillFoliosByPagos(String abreviacionFolio) {
      PagosPoliza result = null;
      Transaction transaction = null;
      try {
         logger.info("Search last generated folio..");

         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();

         String query = "SELECT tap FROM PagosPoliza tap "
                 + "WHERE tap.folio LIKE '" + abreviacionFolio + "%' "
                 + "ORDER BY "
                 + "SUBSTRING_INDEX(tap.folio, '-', 1) DESC, "
                 + "SUBSTRING_INDEX(SUBSTRING_INDEX(tap.folio, '-', -2), '-', -6) DESC ";

         result = (PagosPoliza) session.createQuery(query).setMaxResults(1).uniqueResult();
         transaction.commit();
      } catch (HibernateException e) {
         logger.error("Search for the last generated folio " + e);
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method fillFoliosByPagos() " + e);
         rollback(transaction);
      }
      return result;
   }

   public boolean updatePago(PagosPoliza pago, User user) {
      logger.info("addPagos");
      boolean success = false;
      Transaction transaction = null;

      try {
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();

         pago.setLastUpdatedOn(correcciónDeHr(new Date(), -7));
         pago.setLastUpdatedBy(user);
         session.update(pago);
         transaction.commit();

         success = true;
      } catch (HibernateException e) {
         logger.error("Venta can not be added ", e);
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method addVenta() ", e);
         rollback(transaction);
      }
      return success;
   }

   public boolean createDetellePago(DetellePagoPoliza dpago, User user) {
      logger.info("createDetellePago");
      boolean success = false;
      Transaction transaction = null;

      try {
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         dpago.setId(UUID.randomUUID().toString());
         dpago.setCreatedOn(correcciónDeHr(new Date(), -7));
         dpago.setCreatedBy(user);
         session.save(dpago);
         transaction.commit();

         success = true;
      } catch (HibernateException e) {
         logger.error("Detalle pago can not be added ", e);
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method createDetellePago() ", e);
         rollback(transaction);
      }
      return success;
   }

   public boolean updateDetellePago(DetellePagoPoliza dpago, User user) {
      logger.info("createDetellePago");
      boolean success = false;
      Transaction transaction = null;

      try {
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         dpago.setLastUpdatedOn(correcciónDeHr(new Date(), -7));
         dpago.setLastUpdatedBy(user);
         session.update(dpago);
         transaction.commit();

         success = true;
      } catch (HibernateException e) {
         logger.error("Detalle pago can not be added ", e);
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method createDetellePago() ", e);
         rollback(transaction);
      }
      return success;
   }

   public DetellePagoPoliza getUltimoDetalleByPago(String idPago) {
      logger.info("getUltimoDetalleByPago");

      DetellePagoPoliza result = null;
      Transaction transaction = null;
      try {
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         CriteriaBuilder builder = session.getCriteriaBuilder();

         CriteriaQuery<DetellePagoPoliza> criteria = builder.createQuery(DetellePagoPoliza.class);
         Root<DetellePagoPoliza> root = criteria.from(DetellePagoPoliza.class);

         criteria.where(builder.equal(root.get("pagoPoliza").get("id"), idPago));
         criteria.orderBy(builder.desc(root.get("createdOn")));

         TypedQuery<DetellePagoPoliza> query = session.createQuery(criteria);
         query.setMaxResults(1);
         result = query.getSingleResult();

         transaction.commit();
      } catch (NoResultException e) {
         logger.info("No se encontró detalle para el pago: " + idPago);
         rollback(transaction);
      } catch (HibernateException e) {
         logger.error("Error al obtener el detalle del pago " + idPago, e);
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Error en getUltimoDetalleByPago(" + idPago + ")", e);
         rollback(transaction);
      }
      return result;
   }

   public List<DetellePagoPoliza> fillDetellePagoPolizaByPago(String idPago) {
      logger.info("fillDetellePagoPolizaByPago");
      List<DetellePagoPoliza> resultList = null;
      Transaction transaction = null;
      try {
         logger.info("Searching detalles pagos for poliza..");
         Session session = HibernateUtil.getSessionFactory().getCurrentSession();
         transaction = session.beginTransaction();
         CriteriaBuilder builder = session.getCriteriaBuilder();

         CriteriaQuery<DetellePagoPoliza> criteria = builder.createQuery(DetellePagoPoliza.class);
         Root<DetellePagoPoliza> root = criteria.from(DetellePagoPoliza.class);

         Predicate criterio1 = builder.equal(root.get("pagoPoliza").get("id"), idPago);
         Predicate criterio2 = builder.equal(root.get("estatusActivo"), GenericEnumType.ENABLED);

         criteria.where(builder.and(criterio1, criterio2));
         criteria.orderBy(builder.asc(root.get("createdOn")));

         resultList = session.createQuery(criteria).getResultList();

         logger.info("Total of detalles pagos found: " + (null == resultList ? -1 : resultList.size()));
         transaction.commit();
      } catch (HibernateException e) {
         logger.error("Detalles pagos cannot be loaded ");
         rollback(transaction);
      } catch (Exception e) {
         logger.error("Method fillDetellePagoPolizaByPago ", e);
         rollback(transaction);
      }
      return resultList;
   }

   private Date correcciónDeHr(Date fecha, int horas) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(fecha);
      calendar.add(Calendar.HOUR_OF_DAY, horas);
      Date nuevaFecha = calendar.getTime();
      return nuevaFecha;
   }

}
